/* ccgo: gogame.cc
 *
 * GoGame class definition
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include "gogame.hh"
#include <sstream>
#define P(x, y) ((x) + (y) * z)
#define PL(p) ((p).x + (p).y * z)
#define OTHER_STONE(s) ((s == B_STONE) ? W_STONE : B_STONE)
#define OTHER_TURN(t) ((t == B_TURN) ? W_TURN : B_TURN)
#define TURN_STONE(t) ((t == B_TURN) ? B_STONE : W_STONE)
#define STONE_TURN(s) ((s == B_STONE) ? B_TURN : W_TURN)
#define TURN_OTHER_STONE(t) ((t == B_TURN) ? W_STONE : B_STONE)
#define STONE_OTHER_TURN(s) ((s == B_STONE) ? W_TURN : B_TURN)

using namespace std;

Place::Place()
{
}

Place::Place(int xx, int yy) :
	x(xx),
	y(yy)
{
}

Place::Place(const string & s)
{
	x = s[0] - 'A';
	if (x > 'I' - 'A') -- x;
	y = atoi(s.substr(1).c_str()) - 1;
}

bool Place::operator==(const Place & p) const
{
	return p.x == x && p.y == y;
}

bool Place::operator!=(const Place & p) const
{
	return p.x != x || p.y != y;
}

Place::operator string () const
{
	ostringstream s;
	if (x < 0) return string("--");
	char a = 'A' + x;
	if (a >= 'I') a ++;
	s << a << y + 1;
	return s.str();
}

Place Place::north() const
{
	return Place(x, y + 1);
}

Place Place::south() const
{
	return Place(x, y - 1);
}

Place Place::east() const
{
	return Place(x + 1, y);
}

Place Place::west() const
{
	return Place(x - 1, y);
}


GoGame::GoGame(int zz) :
	z(zz),
	komi(5.5),
	turn(B_TURN),
	b(zz),
	sb(zz),
	c(zz)
{
	t = new Territories[z * z];
	b.fill(NO_STONE);
	sb.fill(NO_STONE);
	capture[B_TURN] = 0;
	capture[W_TURN] = 0;
}

GoGame::~GoGame()
{
	delete[] t;
}

int GoGame::n_moves()
{
	return m.size();
}

void GoGame::add_move(const Move & mm)
{
	m.push_back(mm);
	ko_check(m.back());
	switch (mm.t) {
	case MOVE_PUT:
		b(mm.p) = TURN_STONE(turn);
		for (vector<Place>::const_iterator i = mm.k.begin();
		     i != mm.k.end(); i ++)
			b(* i) = NO_STONE;
		capture[turn] += mm.k.size();
		break;
	case MOVE_HANDICAP:
		for (vector<Place>::const_iterator i = mm.k.begin();
		     i != mm.k.end(); i ++)
			b(* i) = B_STONE;
		break;
	case MOVE_PASS:
		;
	}
	turn = OTHER_TURN(turn);
}

void GoGame::move_put(const Place & p)
{
	Move mm(p, turn);
	get_kill(p, mm.k);
	if (mm.k.size() == 0) {
		if (get_group(p, mm.k) > 0) mm.k.clear();
	}
	add_move(mm);
}

void GoGame::move_pass()
{
	add_move(Move(MOVE_PASS, turn));
}

void GoGame::move_undo()
{
	if (m.size() == 0) return;
	turn = OTHER_TURN(turn);
	switch(m.back().t) {
	case MOVE_PUT:
		BoardStates s;
		if (m.back().k.size() > 0 &&
		    m.back().p == m.back().k[0])
			s = TURN_STONE(turn);
		else s = TURN_OTHER_STONE(turn);
		for (vector<Place>::const_iterator i = m.back().k.begin();
		     i != m.back().k.end(); i ++)
			b(* i) = s;
		b(m.back().p) = NO_STONE;
		capture[turn] -= m.back().k.size();
		break;
	case MOVE_HANDICAP:
		for (vector<Place>::const_iterator i = m.back().k.begin();
		     i != m.back().k.end(); i ++)
			b(* i) = NO_STONE;
		break;
	case MOVE_PASS:
		;
	}
	m.pop_back();
}

const GoGame::Move & GoGame::get_move(int i)
{
	return m[i];
}

int GoGame::get_group(const Place & p, vector<Place> & g)
{
	c.fill(false);
	return qi_group(b(p), p, g);
}

void GoGame::get_kill(const Place & p, vector<Place> & g)
{
	BoardStates bs = TURN_OTHER_STONE(turn);
	if (g.size() > 0) g.clear();

	if (p.x > 0) g.push_back(p.west());
	if (p.x < z - 1) g.push_back(p.east());
	if (p.y > 0) g.push_back(p.south());
	if (p.y < z - 1) g.push_back(p.north());

	for (int nc = g.size(); nc > 0;) {
		Place pp = g.front();
		g.erase(g.begin());
		int bn = g.size();
		c.fill(false);
		c(p) = true;
		int q = qi_group(bs, pp, g);
		nc --;
		for (int i = 0; i < nc; ++ i) if (c(g[i])) {
			g.erase(g.begin() + i);
			i --;
			nc --;
			bn --;
		}
		if (q > 0) g.resize(bn);
	}
}

void GoGame::score_start()
{
	sb = b;
}

void GoGame::score()
{
	c.fill(false);
	for (int x = 0; x < z; x ++) for (int y = 0; y < z; y ++)
		t[P(x, y)] = N_TERRI;
	for (int x = 0; x < z; x ++) for (int y = 0; y < z; y ++) if
		(sb(x, y) == NO_STONE && ! c(x, y)) {
		vector<Place> l;
		bool wo = false;
		bool bo = false;
		l.clear();
		l.push_back(Place(x, y));
		terri_group(l, wo, bo);
		if (wo && ! bo) for (vector<Place>::const_iterator
					     i = l.begin();
				     i != l.end(); i ++) t[PL(* i)] = W_TERRI;
		if (! wo && bo) for (vector<Place>::const_iterator
					     i = l.begin();
				     i != l.end(); i ++) t[PL(* i)] = B_TERRI;
	}
	c.fill(false);
	bool rr;
	do {
		rr = false;
		c.fill(false);
		for (int x = 0; x < z; x ++) for (int y = 0; y < z; y ++) if
			(! c(x, y) && sb(x, y) == B_STONE) {
				vector<Place> l;
			vector<Place> q;
			hang_group(sb(x, y), Place(x, y), l, q);
			if (q.size() == 1) {
				t[PL(q[0])] = B_FILL;
				rr = true;
			}
			while (q.size() > 0) {
				c(q.back()) = false;
				q.pop_back();
			}
		}
		c.fill(false);
		for (int x = 0; x < z; x ++) for (int y = 0; y < z; y ++) if
			(! c(x, y) && sb(x, y) == W_STONE) {
			vector<Place> l;
			vector<Place> q;
			hang_group(sb(x, y), Place(x, y), l, q);
			if (q.size() == 1) {
				t[PL(q[0])] = W_FILL;
				rr = true;
			}
			while (q.size() > 0) {
				c(q.back()) = false;
				q.pop_back();
			}
		}
	} while (rr);
}

void GoGame::rid_group(const Place & p)
{
	if (sb(p) == NO_STONE) return;
	vector<Place> l;
	c.fill(false);
	qi_group(sb(p), p, l);
	capture[STONE_OTHER_TURN(sb(p))] += l.size();
	for (vector<Place>::const_iterator i = l.begin();
	     i != l.end(); i ++)
		sb(* i) = NO_STONE;
}

void GoGame::restore()
{
	b.fill(NO_STONE);
	capture[B_TURN] = 0;
	capture[W_TURN] = 0;
	for (vector<Move>::const_iterator i = m.begin(); i != m.end(); i ++)
		switch (i->t) {
		case MOVE_PUT:
			b(i->p) = TURN_STONE(i->tn);
			for (vector<Place>::const_iterator j = i->k.begin();
			     j != i->k.end(); j ++)
				b(* j) = NO_STONE;
			capture[i->tn] += i->k.size();
			break;
		case MOVE_HANDICAP:
			for (vector<Place>::const_iterator j = i->k.begin();
			     j != i->k.end(); j ++)
				b(* j) = B_STONE;
			break;
		case MOVE_PASS:
			;
		}
	sb = b;
}

GoGame::Territories GoGame::terri(const Place & p)
{
	return t[PL(p)];
}

void GoGame::set_komi(float k)
{
	komi = k;
}

int GoGame::qi_group(BoardStates bs, const Place & p, vector<Place> & g)
{
	if (p.x < 0 || p.x >= z || p.y < 0 || p.y >= z || c(p)) return 0;
	c(p) = true;
	if (bs == NO_STONE) bs = TURN_STONE(turn);
	else {
		if (b(p) == OTHER_STONE(bs)) return 0;
		if (b(p) == NO_STONE) return 1;
		g.push_back(p);
	}
	int q = 0;
	q += qi_group(bs, p.west(), g);
	q += qi_group(bs, p.east(), g);
	q += qi_group(bs, p.south(), g);
	q += qi_group(bs, p.north(), g);
	return q;
}

void GoGame::ko_check(Move & mm)
{
	mm.ko = false;
	if (mm.k.size() == 1 && mm.p != mm.k[0]) {
		vector<Place> l;
		if (get_group(mm.p, l) == 0 && l.size() == 0)
			mm.ko = true;
	}
}

void GoGame::terri_group(vector<Place> & g, bool & ww, bool & bb)
{
	Place gb = g.back();
	Place p;
	c(gb) = true;
	p = gb.west();
	if (p.x >= 0 && ! c(p)) {
		if (sb(p) == NO_STONE) {
			g.push_back(p);
			terri_group(g, ww, bb);
		} else {
			if (sb(p) == W_STONE) ww = true;
			else bb = true;
		}
	}
	p = gb.east();
	if (p.x < z && ! c(p)) {
		if (sb(p) == NO_STONE) {
			g.push_back(p);
			terri_group(g, ww, bb);
		} else {
			if (sb(p) == W_STONE) ww = true;
			else bb = true;
		}
	}
	p = gb.south();
	if (p.y >= 0 && ! c(p)) {
		if (sb(p) == NO_STONE) {
			g.push_back(p);
			terri_group(g, ww, bb);
		} else {
			if (sb(p) == W_STONE) ww = true;
			else bb = true;
		}
	}
	p = gb.north();
	if (p.y < z && ! c(p)) {
		if (sb(p) == NO_STONE) {
			g.push_back(p);
			terri_group(g, ww, bb);
		} else {
			if (sb(p) == W_STONE) ww = true;
			else bb = true;
		}
	}
}

void GoGame::hang_group(BoardStates bs, const Place & p, vector<Place> & g,
			vector<Place> & q)
{
	if (p.x < 0 || p.x >= z || p.y < 0 || p.y >= z || c(p)) return;
	if (t[PL(p)] == ((bs == B_STONE) ? B_TERRI : W_TERRI)) {
		q.push_back(p);
		c(p) = true;
		return;
	}
	if (sb(p) == ((bs == B_STONE) ? W_STONE : B_STONE)) return;
	g.push_back(p);
	c(p) = true;
	hang_group(bs, p.west(), g, q);
	hang_group(bs, p.east(), g, q);
	hang_group(bs, p.south(), g, q);
	hang_group(bs, p.north(), g, q);
}
