/* ccgo: gotime.cc
 *
 * GoTime class definition
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include "gotime.hh"
#include "gowin.hh"
#include <sstream>

GoTime::GoTime(GoWin & gg) :
	g(gg)
{
	pack_start(w_timing);
	w_timing.show();
	pack_start(b_timing);
	b_timing.show();
	tc = Gtk::Main::timeout.connect(slot(this, & GoTime::time_down), 1000);
	running = false;
	main_time = 600;
	byo_time = 600;
	byo_stones = 25;
}

GoTime::~GoTime()
{
	tc.disconnect();
}

void GoTime::set_time(int wt, int wb, int bt, int bb)
{
	wtime = wt;
	wbyo = wb;
	btime = bt;
	bbyo = bb;
	update_timing();
}

void GoTime::set_time(const GameTime & t)
{
	wtime = t.wt;
	wbyo = t.wb;
	btime = t.bt;
	bbyo = t.bb;
	update_timing();
}

void GoTime::start()
{
	running = true;
	wtime = main_time;
	btime = main_time;
	wbyo = - 1;
	bbyo = - 1;
	update_timing();
}

void GoTime::stop()
{
	running = false;
}

void GoTime::cont()
{
	running = true;
}

gint GoTime::time_down()
{
	if (! running) return true;
	if (g.turn == GoWin::B_TURN) {
		if (btime > 0) -- btime;
		else if (bbyo > 0) timeup.emit();
		else {
			bbyo = byo_stones;
			btime = byo_time;
		}
	} else {
		if (wtime > 0) -- wtime;
		else if (wbyo > 0) timeup.emit();
		else {
			wbyo = byo_stones;
			wtime = byo_time;
		}
	}
	update_timing();
	return true;
}

void GoTime::realize_impl()
{
	Gtk::VBox::realize_impl();
	adj_sizes();
}

void GoTime::style_changed_impl(Gtk::Style * s)
{
	Gtk::VBox::style_changed_impl(s);
	adj_sizes();
}

void GoTime::update_timing()
{
	std::ostringstream l;
	int s = wtime % 60;
	l << "W:" << wtime / 60 << ((s > 9) ? ":" : ":0") << s << "/";
	if (wbyo < 10) l << ' ';
	if (wbyo > 0) l << wbyo;
	else l << "-";
	w_timing.set_text(l.str());
	l.str("");
	s = btime % 60;
	l << "B:" << btime / 60 << ((s > 9) ? ":" : ":0") << s << "/";
	if (bbyo < 10) l << ' ';
	if (bbyo > 0) l << bbyo;
	else l << "-";
	b_timing.set_text(l.str());
}

void GoTime::switching()
{
	if (g.turn == GoGame::W_TURN) {
		-- wbyo;
		if (wbyo == 0) {
			wbyo = byo_stones;
			wtime = byo_time;
		}
	} else {
		-- bbyo;
		if (bbyo == 0) {
			bbyo = byo_stones;
			btime = byo_time;
		}
	}
}

void GoTime::set_byotime(int t)
{
	byo_time = t * 60;
}

void GoTime::adj_sizes()
{
	w_timing.set_usize(w_timing.get_style()->get_font().string_width
			   ("W:44:44/44"), -1);
	b_timing.set_usize(b_timing.get_style()->get_font().string_width
			   ("B:44:44/44"), -1);	
}
