/* ccgo: gowin.hh
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef GOWIN_HH
#define GOWIN_HH
#include <map>
#include <sigc++/object.h>
#include <gtk--.h>
#include "goscore.hh"
#include "gotime.hh"
#include "goboard.hh"
#include "gotalk.hh"
using namespace std;
class GoWin :
	virtual public Gtk::Window,
	virtual public GoGame
{
public:
	GoWin(int);
	~GoWin();
	SigC::Signal1<void, const string &> talk;
	virtual void set_komi(float);
	virtual void set_byotime(int);
protected:
	// GoGame g;
	GoBoard bd;
	std::string id;
	std::string wp;
	std::string bp;

	enum MatchStages {
		STAGE_PLAYING,
		STAGE_SCORING,
		STAGE_END
	} ms;

	Gtk::Label numm;
	Gtk::Adjustment dma;
	Gtk::SpinButton dms;
	Gtk::Label lastm;
	int dm;
	void n_move_change();
	void d_move_change();
	void change_display();
	virtual void back_display();
	virtual void fore_display();
	GoScore gs;
	friend class GoTime;
	GoTime gt;
	Gtk::HBox top_area;
	Gtk::VBox button_area;
	GoTalk gk;
	void duplicate();
private:
	Gtk::Label mouse_position;
	void mouse_pose(const Place &);
};

class GoWinObserve :
	virtual public GoWin
{
public:
	GoWinObserve(int);
	~GoWinObserve();
	virtual void add_move(const Move &);
	virtual void undo_move();
	void set_id(const string &);
	const string & get_id();
	void set_white(const string &);
	const string & get_white();
	void set_black(const string &);
	const string & get_black();
	void set_status(const string &);
	virtual void set_end();
	void set_time(int, int, int, int);
	void set_time(const GameTime &);
	void set_headline(const string &);
	void set_board(const int *);
	void kibitz(const string &);
protected:
private:
	string id_str;
	string white;
	string black;
	Gtk::Label status_bar;
	int magic;
};

class GoWinPlay :
	virtual public GoWin
{
public:
	GoWinPlay(int = 19);
	~GoWinPlay();
	int pass_count;
	virtual void score_display();
	virtual void enter_score();
	virtual void restore_board();
	virtual void remove_group(const Place &);
protected:
	virtual void hit_board(const Place &);
	virtual void hit_pass();
	virtual void hit_undo();
	virtual void fore_display();
	virtual void back_display();
	Gtk::Button button_pass;
	Gtk::Button button_undo;
private:
	Gtk::Label pass_l;
	int wdame;
	int bdame;
};

class GoWinMatch :
	virtual public GoWinObserve,
	virtual public GoWinPlay
{
public:
	GoWinMatch(int, bool, bool);
	~GoWinMatch();
	virtual void add_move(const Move &);
	virtual void undo_move();
	virtual void set_end();
	SigC::Signal1<void, const Place &> board_hit;
	SigC::Signal0<void> pass_hit;
	SigC::Signal0<void> undo_hit;
	SigC::Signal0<void> done_hit;
protected:
	virtual void hit_board(const Place &);
	virtual void hit_pass();
	virtual void hit_undo();
private:
	std::map<Turns, bool> play_turn;
};
#endif /* GOWIN_HH */
