/* ccgo: igsconsole.cc
 *
 * IgsConsole class definition
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include "igsconsole.hh"

using namespace std;

IgsCommand::IgsCommand() :
	Gtk::Entry()
{
	ri = hist.rbegin();
}

void IgsCommand::clear_history()
{
	hist.clear();
	ri = hist.rbegin();
}

gint IgsCommand::key_press_event_impl(GdkEventKey * e)
{
	switch (e->keyval) {
	case GDK_Up:
		if (ri == hist.rend()) break;
		if (ri == hist.rbegin()) current_line = get_text();
		set_text(* ri);
		++ ri;
		break;
	case GDK_Down:
		if (ri == hist.rbegin()) break;
		-- ri;
		if (ri == hist.rbegin()) set_text(current_line);
		else set_text(* (ri - 1));
		break;
	default:
		return Gtk::Entry::key_press_event_impl(e);
	}
	return true;
}

void IgsCommand::activate_impl()
{
	Gtk::Entry::activate_impl();
	hist.push_back(get_text());
	if (hist.size() > 100) hist.erase(hist.begin());
	for (vector<string>::iterator i = hist.begin();
	     i + 1 != hist.end(); i ++) {
		if (* i == hist.back()) {
			hist.erase(i);
			break;
		}
	}
	ri = hist.rbegin();
	current_line = "";
	set_text(current_line);
}

IgsConsole::IgsConsole() :
	prompt("")
{
	Gtk::ScrolledWindow * sw;
	Gtk::HBox * h;
	sw = manage(new Gtk::ScrolledWindow());
	sw->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	sw->add(text_dump);
	sw->show();
	pack_start(* sw);

	// Gdk_Font f("-*-lucidatypewriter-medium-r-*-*-*-100-*-*-*-*-*-*");
	ct_fixed = text_dump.get_context();
	ct_system = text_dump.get_context();
	ct_yell = text_dump.get_context();
	ct_file = text_dump.get_context();
	Gdk_Font f("fixed");
	ct_fixed.set_font(f);
	ct_file.set_font(f);
	ct_system.set_foreground(Gdk_Color("red"));
	ct_yell.set_foreground(Gdk_Color("green"));
	ct_file.set_foreground(Gdk_Color("blue"));
// 	Gtk::Style * s = text_dump.get_style()->copy();
// 	s->set_font(f);
// 	text_dump.set_style(* s);
	text_dump.set_usize(f.char_width('A') * 80, f.char_height('|') * 32);

// 	text_dump.style_changed.connect(slot(this, & IgsConsole::style_fix));
	text_dump.set_line_wrap(false);
	text_dump.show();
	h = manage(new Gtk::HBox());
	h->pack_start(prompt, false);
	prompt.show();
	h->pack_end(command_line);
	command_line.activate.connect(slot(this, & IgsConsole::enter_line));
	command_line.show();
	h->show();
	pack_end(* h, false);
	text_dump.unset_flags(GTK_CAN_FOCUS);
}

IgsConsole::~IgsConsole()
{
}

void IgsConsole::dump(const string & s, TextType t)
{
	switch (t) {
	case TT_NORMAL:
		text_dump.insert(s + "\n");
		break;
	case TT_FIXED:
		text_dump.insert(ct_fixed, s + "\n");
		break;
	case TT_SYSTEM:
		text_dump.insert(ct_system, s + "\n");
		break;
	case TT_YELL:
		text_dump.insert(ct_yell, s + "\n");
		break;
	case TT_FILE:
		text_dump.insert(ct_file, s + "\n");
		break;
	}
}

void IgsConsole::set_prompt(const string & s)
{
	prompt.set_text(s);
}

void IgsConsole::clear_history()
{
	command_line.clear_history();
}

void IgsConsole::set_visible(bool b)
{
	command_line.set_visibility(b);
}

void IgsConsole::set_input_active(bool b)
{
	command_line.set_sensitive(b);
}

void IgsConsole::enter_line()
{
	string s = command_line.get_text();
	enter.emit(s);
}

// void IgsConsole::style_fix(Gtk::Style *)
// {
// 	Gtk::Style * s = text_dump.get_style();
// 	text_dump.set_usize(s->get_font().char_width('a') * 80,
// 			    s->get_font().char_height('|') * 24);
// }
